#!/usr/bin/env python

'''HTML parser for creating html-stripped versions of web pages.

Based on HTMLParser, but different in handling of links.  Link hrefs are
spelled out within parens immediately after the link text, and only if the
href is different than the link text. Thus:

	<href://www.python.org>The python web site</a>
		==> "The python web site (href://www.python.org)"

    but 

	<href://www.python.org>http://www.python.org</a>
		==> "href://www.python.org"

This should be useful for, eg, automatically translating combined web/ftp
site README.html files to tidy README files.'''

__version__ = "1.0"
__author__ = "Ken Manheimer, klm@python.org"

from htmllib import HTMLParser

class HTMLInLineParser(HTMLParser):
    '''A version of the html parser that spells out link values inline.

    Link hrefs are spelled out within parens immediately after the link
    text, and only if the href is different than the link text.'''

    # Mostly a hack to preserve the link text, which probably should be
    # implemented in the underlying HTMLParser class...

    def __init__(self, formatter, verbose=0):
	HTMLParser.__init__(self, formatter, verbose=0)
    def anchor_bgn(self, href, name, type):
	self.save_bgn()
	HTMLParser.anchor_bgn(self, href, name, type)
    def anchor_end(self):
	text = self.save_end()
	self.handle_data(text)
	if self.anchor:
	    if text != self.anchor:
		self.handle_data(" (%s)" % self.anchor)
	    self.anchor = None


def main(args = None):
    '''Apply HTMLInLineParser to a file, or stdin if no arguments specified.
    
    Leading '-s' option means be very silent - parse, but produce no output.'''
    import sys, formatter

    if not args:
	args = sys.argv[1:]

    silent = args and args[0] == '-s'
    if silent:
	del args[0]

    if args:
	file = args[0]
    else:
	file = '-'

    if file == '-':
	f = sys.stdin
    else:
	try:
	    f = open(file, 'r')
	except IOError, msg:
	    print file, ":", msg
	    sys.exit(1)

    data = f.read()

    if f is not sys.stdin:
	f.close()
    
    if silent:
	f = formatter.NullFormatter()
    else:
	f = formatter.AbstractFormatter(formatter.DumbWriter())

    p = HTMLInLineParser(f)
    p.feed(data)
    p.close()


if __name__ == '__main__':
    main()
